/*
 * getopt-exp.c, Dietrich Bollmann, Tokyo 2008/04/01
 *
 * GNU getopt experiments...
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* using the getopt() from the GNU C Library */
/*
  #include <getopt.h>
*/

/* using local copy of GNU getopt() */
#include "getopt.h"

/* some option definitions */
static struct option longopts[] = {
	{ "help",          no_argument,       NULL, 'h' },
	{ "no-arg",        no_argument,       NULL, 'n' },
	{ "arg-required",  required_argument, NULL, 'r' },
	{ "arg-optional",  optional_argument, NULL, 'o' },
	{ NULL,            0,                 NULL, 0   }
};

/**
   help function.
*/
void print_usage()
{
	printf("usage: ./getopt-exp [options]\n");
	printf("\n");
	printf("options:\n");
	printf("\n");
	printf("  -h | --help            - print this help message.\n");
	printf("  -n | --no-arg          - an option without arguments.\n");
	printf("  -r | --arg-required    - an option with a required argument.\n");
	printf("  -o | --arg-optional    - an option with an optional argument.\n");
	printf("\n");
	printf("examples:\n");
	printf("\n");
	printf("  ./getopt-exp -h\n");
	printf("  ./getopt-exp --help\n");
	printf("  ./getopt-exp -n\n");
	printf("  ./getopt-exp --no-arg\n");
	printf("  ./getopt-exp -n --no-arg\n");
	printf("  ./getopt-exp -r arg1 --arg-required arg2\n");
	printf("  ./getopt-exp -rarg1 --arg-required=arg2\n");
	printf("  ./getopt-exp -o --arg-optional\n");
	printf("  ./getopt-exp -oarg3 --arg-optional=arg4\n");
	printf("\n");
	printf("  ./getopt-exp \\\n");
	printf("    one \\\n");
	printf("    -n --no-arg \\\n");
	printf("    two \\\n");
	printf("    -r arg1 --arg-required arg2 -rarg1 --arg-required=arg2 \\\n");
	printf("    three \\\n");
	printf("    -o --arg-optional -oarg3 --arg-optional=arg4 \\\n");
	printf("    four \\\n");
	printf("    -u --unknown-option \\\n");
	printf("    five \\\n");
	printf("    -u --unknown-option \\\n");
	printf("    six\n");
	printf("\n");
	exit(0);
}

/**
   dump command line arguments
*/
void dump_command_line_arguments(int argc, char *argv[])
{
	printf("* dumping command line options:\n");
	printf("\n");
	printf("- number of arguments: %d\n", (argc - 1));
	printf("\n");
	int i; 
	for (i = 1; i < argc; i++) {
		printf("  - argv[%d]: %s\n", i, argv[i]);
	}
	printf("\n");
}

/**
   external 'getopt' variables

   from http://www.gnu.org/software/libtool/manual/libc/Using-Getopt.html

   - char * optarg

   This variable is set by getopt to point at the value of the
   option argument, for those options that accept arguments. 

   - int optind

   This variable is set by getopt to the index of the next element
   of the argv array to be processed. Once getopt has found all of
   the option arguments, you can use this variable to determine
   where the remaining non-option arguments begin. The initial value
   of this variable is 1. 

   - int optopt

   When getopt encounters an unknown option character or an option
   with a missing required argument, it stores that option character
   in this variable. You can use this for providing your own
   diagnostic messages. 
*/
extern char* optarg;
extern int optind;
extern int optopt;

/**
   parsing the command line arguments with getopt
*/
void parsing_command_line_arguments(int argc, char *argv[])
{
	printf("* parsing the command line parameters with getopt:\n");
	printf("\n");

	int c;
	while ((c = getopt_long(argc, argv, "hnr:o::", longopts, NULL)) != -1) {

		switch(c) {

		case 'h': print_usage();                                              break;
		case 'n': printf("  - no-arg option found.\n");                       break;
		case 'r': printf("  - arg-required option found; arg: %s\n", optarg); break;
		case 'o': printf("  - arg-optional option found; arg: %s\n", optarg); break;

		case '?':
			if (optopt == 'c')
				fprintf (stderr, "  - option -%c requires an argument.\n", optopt);
			else if (isprint (optopt))
				fprintf (stderr, "  - unknown option `-%c'.\n", optopt);
			else
				fprintf (stderr, "  - unknown option character `\\x%x'.\n", optopt);
			break;

		default: /* ignore */ break;
		}
	}

	/*
	  printing the non-option arguments

	  the arguments which are not recognized as options
	  are sorted to the end of argv
	  and start at index `optind'.
	*/
	printf("  non-option arguments: ");
	int i;
	for (i = optind; i < argc; i++) {
		if (i > optind) printf (", ");
		printf ("%s", argv[i]);
	}
	printf(".\n", argv[i]);
	printf("\n");
}

/**
   main function.
*/
int main(int argc, char *argv[])
{
	printf("GNU getopt() experiments :)\n");
	printf("\n");

	/* dump command line arguments */
	dump_command_line_arguments(argc, argv);

	/* parsing the command line arguments with getopt */
	parsing_command_line_arguments(argc, argv);

	/* dump command line arguments */
	dump_command_line_arguments(argc, argv);

	/* done */
	printf("bye...\n");
	printf("\n");
	exit(0);
}

/* fin */
